module myproblem

implicit none
	
	! LOCAL SCALARS
	integer, parameter :: problem = 1

  ! SUBROUTINES
  public :: inip, evalf, evalg, quadstep

contains

	subroutine inip(m,quadratic,stps)

	implicit none

	! SCALAR ARGUMENTS
	integer, intent(out) :: m
	
	! ARRAY ARGUMENTS
	logical, allocatable, intent(out) :: quadratic(:)
	real(kind=8),intent(out) :: stps(4)
	
	! LOCAL SCALARS
	integer :: i
	real(kind=8), parameter :: zero = 0.0d0, one = 1.0d0
	
	if ( problem == 1 ) then
			m = 2		

			allocate(quadratic(m))	
			quadratic(1) = .false.
			quadratic(2) = .false.
	
	stps(1) = 1.0d-3
	stps(2) = 3.0d0
	stps(3) = 1.0d1
	stps(4) = 1.0d3
	
	return
	
	end if
	 
	! --------------------------------------------------------------------	 
	
	if ( problem == 2 ) then
			m = 2		

			allocate(quadratic(m))	
			quadratic(1) = .false.
			quadratic(2) = .true.

	stps(1) = 1.0d-3
	stps(2) = 0.5d0
	stps(3) = 2.5d0
	stps(4) = 1.0d3
	
	return
	
	end if
	
! --------------------------------------------------------------------	 
	
	if ( problem == 3 ) then
			m = 2		

			allocate(quadratic(m))	
			quadratic(1) = .false.
			quadratic(2) = .false.

	stps(1) = 1.0d-3
	stps(2) = 0.4d0
	stps(3) = 0.6d0
	stps(4) = 1.0d3
	
	return
	
	end if
	
! --------------------------------------------------------------------	 
	
	if ( problem == 4 ) then
			m = 3		

			allocate(quadratic(m))	
			quadratic(1) = .false.
			quadratic(2) = .false.
			quadratic(3) = .true.
			
			stps(1) = 1.0d-3
			stps(2) = 0.25d0
			stps(3) = 0.5d0
			stps(4) = 1.0d3
	
	return
	end if
	
! --------------------------------------------------------------------	 
	
	if ( problem == 5 ) then
			m = 2		

			allocate(quadratic(m))	
			quadratic(1) = .false.
			quadratic(2) = .false.
			
			stps(1) = 1.0d-3
			stps(2) = 0.3d0
			stps(3) = 1.5d0
			stps(4) = 1.0d3
	
	return
	end if
	
! --------------------------------------------------------------------	 
	
	if ( problem == 6 ) then
			m = 2		

			allocate(quadratic(m))	
			quadratic(1) = .false.
			quadratic(2) = .false.
			
			stps(1) = 1.0d-3
			stps(2) = 0.2d0
			stps(3) = 1.0d1
			stps(4) = 1.0d3
	
	return
	end if		
	
! --------------------------------------------------------------------	 
	
	if ( problem == 7 ) then
			m = 8		

			allocate(quadratic(m))	
			quadratic(1) = .false.
			quadratic(2) = .false.
			quadratic(3) = .false.
			quadratic(4) = .false.
			quadratic(5) = .false.
			quadratic(6) = .false.
			quadratic(7) = .false.
			quadratic(8) = .false.
			
			stps(1) = 1.0d-3
			stps(2) = 1.0d-1
			stps(3) = 1.0d1
			stps(4) = 1.0d3
	
	return
	end if	

	end subroutine inip

	!***********************************************************************
	!***********************************************************************

	subroutine evalf(x,f,ind)
	
	implicit none

	! SCALAR ARGUMENTS
	integer, intent(in) :: ind
	real(kind=8), intent(in) :: x
	real(kind=8), intent(out) :: f
	
	! LOCAL SCALARS
	real(kind=8), parameter :: one = 1.0d0, two = 2.0d0
	real(kind=8) :: t,a,b
		
	if ( problem == 1 ) then
			if ( ind .eq. 1 ) then
					a = 1.6d-1
					f = -x/(x**2+a)
					return
			elseif ( ind .eq. 2 ) then
					a = 4.0d-3
					t = x + a
					f = t**5 - two*t**4
					return
			end if
			return
	end if
	
	! --------------------------------------------------------------------
	
	if ( problem == 2 ) then
			if (ind .eq. 1) then
					if ( x < 0 ) then
							f = -1.0d2 * x + 1.0d4 * x ** 2
					elseif ( x >= 0.0d0 .and. x <= 1.0d0 ) then
							f = -log(1.0d0 + 1.0d2 * x )
					else
							f = -log(1.01d2) -1.0d2/1.01d2 * ( x - 1.0d0 ) +         &
							                     1.0d2**2/1.01d2**2 * ( x - 1.0d0 )**2
					end if
					return
			elseif (ind .eq. 2) then
					f = 1.0d-1 * x**2 - x
					return
			end if
	end if
	
  ! --------------------------------------------------------------------
	
	if ( problem == 3 ) then	
			if ( ind .eq. 1 ) then
					a = 1.0d-2
					b = 1.0d-3
			elseif ( ind .eq. 2 ) then
					a = 1.0d-3
					b = 1.0d-2
			end if
			f = ( sqrt(one + a**2) - a  ) * sqrt( (one-x)**2 + b**2 ) +  &
							( sqrt(one + b**2) - b  ) * sqrt( x**2 + a**2 )
			f = 1.0d3 * f
	end if
	
	! --------------------------------------------------------------------
	
	if ( problem == 4 ) then
			if ( ind .eq. 1 ) then
					a = 1.6d-1
					f = -x/(x**2+a)
					return
			elseif ( ind .eq. 2 ) then
					f = two - 0.8d0 * exp(-((x-0.6d0)/0.4d0)**2) - exp(-((x-0.2d0)/0.4d-1)**2)
					return
			elseif ( ind .eq. 3 ) then	
					f = 1.0d-1 * x**2 - x
			end if
	end if
	
	! --------------------------------------------------------------------
	
	if ( problem == 5 ) then
			if ( ind .eq. 1 ) then
					a = 4.0d-3
					t = x + a
					f = t**5 - two*t**4
					return
			elseif ( ind .eq. 2 ) then
					a = 1.0d1
					f = exp(- a * x )
					return
			end if
	end if
	
! --------------------------------------------------------------------
	
	if ( problem == 6 ) then
			if (ind .eq. 1) then
					a = 1.0d1
					f = exp( -a * x )
					return
			elseif (ind .eq. 2) then
					if ( x == 0 ) then
							f = 0
					else
							a = 1.0d3
							f = - x + a * x**3 * sin( 1/x )
					end if
					return
			end if
	end if			
	
	! --------------------------------------------------------------------	
	
	if ( problem == 7 ) then
			if ( ind .eq. 1 ) then
					a = 1.6d-1
					f = -x/(x**2+a)
					return
			elseif ( ind .eq. 2 ) then
					a = 4.0d-3
					t = x + a
					f = t**5 - two*t**4
					return
			elseif ( ind .eq. 3 ) then
					if ( x < 0 ) then
							f = -1.0d2 * x + 1.0d4 * x ** 2
					elseif ( x >= 0.0d0 .and. x <= 1.0d0 ) then
							f = -log(1.0d0 + 1.0d2 * x )
					else
							f = -log(1.01d2) -1.0d2/1.01d2 * ( x - 1.0d0 ) +         &
																	 1.0d2**2/1.01d2**2 * ( x - 1.0d0 )**2
					end if
					return
			elseif ( ind .eq. 4 ) then
					a = 1.0d-2
					b = 1.0d-3
					f = ( sqrt(one + a**2) - a  ) * sqrt( (one-x)**2 + b**2 ) +  &
							( sqrt(one + b**2) - b  ) * sqrt( x**2 + a**2 )
					return
			elseif ( ind .eq. 5 ) then
					a = 1.0d-3
					b = 1.0d-2
					f = ( sqrt(one + a**2) - a  ) * sqrt( (one-x)**2 + b**2 ) +  &
							( sqrt(one + b**2) - b  ) * sqrt( x**2 + a**2 )
					return
			elseif ( ind .eq. 6 ) then		
					f = two - 0.8d0 * exp(-((x-0.6d0)/0.4d0)**2) - exp(-((x-0.2d0)/0.4d-1)**2)
					return
			elseif ( ind .eq. 7 ) then				
					a = 1.0d1
					f = exp(- a * x )
					return
			elseif ( ind .eq. 8 ) then
					if ( x == 0 ) then
							f = 0
					else
							a = 1.0d3
							f = - x + a * x**3 * sin( 1/x )
					end if
					return		
			end if	
			
	end if

	end subroutine evalf

	!***********************************************************************
	!***********************************************************************

	subroutine evalg(x,g,ind)
	
	implicit none
	
	! SCALAR ARGUMENTS
	integer, intent(in) :: ind
	real(kind=8), intent(in) :: x
	real(kind=8), intent(out) :: g
	
	! LOCAL SCALARS
	real(kind=8), parameter :: one = 1.0d0, two=2.0d0
	real(kind=8) :: t, a, b
	
	if ( problem == 1 ) then
			if ( ind .eq. 1 ) then
				 a = 1.6d-1
				 g = (x**2-a)/(x**2+a)**2
				 return
			elseif ( ind .eq. 2 ) then
				 a = 4.0d-3
				 t = x + a
				 g = 5*t**4 - 4*two*t**3
				 return
			end if
	end if	
	
  ! --------------------------------------------------------------------
	
	if ( problem == 2 ) then
			if ( ind .eq. 1 ) then
					if ( x < 0 ) then
							g = -1.0d2 + 2.0d0 * 1.0d4 * x
					elseif ( x >= 0.0d0 .and. x <= 1.0d0 ) then
							g = -1.0d2 / (1.0d0 + 1.0d2 * x )
					else
							g = -1.0d2/1.01d2 + 2.0d0 * 1.0d2**2/1.01d2**2 * ( x - 1.0d0 )
					end if
					return
			elseif ( ind .eq. 2 ) then
					g = 2.0d0 * 1.0d-1 * x - 1.0d0
					return
			end if
			return
	end if
	
	! --------------------------------------------------------------------
	
	if ( problem == 3 ) then	
			if ( ind .eq. 1 ) then
					a = 1.0d-2
					b = 1.0d-3
			elseif ( ind .eq. 2 ) then
					a = 1.0d-3
					b = 1.0d-2
			end if
			g = -(one-x) * ( sqrt(one + a**2) - a  ) * ( (one-x)**2 + b**2 ) ** (-0.5d0) +  &
					  x * ( sqrt(one + b**2) - b  ) * ( x**2 + a**2 ) ** (-0.5d0)
			g = 1.0d3 * g
	end if
	
! ----------------------------------------------------------------------
	
	if ( problem == 4 ) then
			if ( ind .eq. 1 ) then
					a = 1.6d-1
				  g = (x**2-a)/(x**2+a)**2
					return
			elseif ( ind .eq. 2 ) then
					g = 1.0d1 * (x-0.6d0) * exp(-((x-0.6d0)/0.4d0)**2) +         &
							1.25d3 * (x-0.2d0) * exp(-((x-0.2d0)/0.4d-1)**2)
					return
			elseif ( ind .eq. 3 ) then		
					g = 2.0d0 * 1.0d-1 * x - 1.0d0
			end if
	end if		
	
! --------------------------------------------------------------------
	
	if ( problem == 5 ) then
			if ( ind .eq. 1 ) then
					a = 4.0d-3
					t = x + a
					g = 5*t**4 - 4*two*t**3
					return
			elseif ( ind .eq. 2 ) then
					a = 1.0d1
					g = -a * exp(- a * x )
					return
			end if
	end if	
	
! --------------------------------------------------------------------
	
	if ( problem == 6 ) then
			if (ind .eq. 1) then
					a = 1.0d1
					g = -a * exp( -a * x )
					return
			elseif (ind .eq. 2) then
					if ( x == 0 ) then
							g = -1.0d0
					else
							a = 1.0d3
							g = -1.0d0 + a * 3.0d0 * x**2 * sin( 1/x ) - a * x * cos( 1/x )
					end if
					return
			end if
	end if			
	
! --------------------------------------------------------------------
	
	if ( problem == 7 ) then
			if ( ind .eq. 1 ) then
				 a = 1.6d-1
				 g = (x**2-a)/(x**2+a)**2
				 return
			elseif ( ind .eq. 2 ) then
				 a = 4.0d-3
				 t = x + a
				 g = 5*t**4 - 4*two*t**3
				 return
			elseif ( ind .eq. 3 ) then
				 if ( x < 0 ) then
							g = -1.0d2 + 2.0d0 * 1.0d4 * x
					elseif ( x >= 0.0d0 .and. x <= 1.0d0 ) then
							g = -1.0d2 / (1.0d0 + 1.0d2 * x )
					else
							g = -1.0d2/1.01d2 + 2.0d0 * 1.0d2**2/1.01d2**2 * ( x - 1.0d0 )
					end if
					return
			elseif ( ind .eq. 4 ) then
					a = 1.0d-2
					b = 1.0d-3
					g = -(one-x) * ( sqrt(one + a**2) - a  ) * ( (one-x)**2 + b**2 ) ** (-0.5d0) +  &
					  x * ( sqrt(one + b**2) - b  ) * ( x**2 + a**2 ) ** (-0.5d0)
					return  
			elseif ( ind .eq. 5 ) then
					a = 1.0d-3
					b = 1.0d-2
					g = -(one-x) * ( sqrt(one + a**2) - a  ) * ( (one-x)**2 + b**2 ) ** (-0.5d0) +  &
					  x * ( sqrt(one + b**2) - b  ) * ( x**2 + a**2 ) ** (-0.5d0)
					return  
			elseif ( ind .eq. 6 ) then
					g = 1.0d1 * (x-0.6d0) * exp(-((x-0.6d0)/0.4d0)**2) +         &
							1.25d3 * (x-0.2d0) * exp(-((x-0.2d0)/0.4d-1)**2)
					return
			elseif ( ind .eq. 7 ) then		
					a = 1.0d1
					g = -a * exp(- a * x )
					return
			elseif ( ind .eq. 8 ) then
					if ( x == 0 ) then
							g = -1.0d0
					else
							a = 1.0d3
							g = -1.0d0 + a * 3.0d0 * x**2 * sin( 1/x ) - a * x * cos( 1/x )
					end if
					return		
			end if			
	end if	

	end subroutine evalg

	!***********************************************************************
	!***********************************************************************

	subroutine quadstep(x,ind,flag)
	
	implicit none

	! SCALAR ARGUMENTS
	integer, intent(in) :: ind
	integer, intent(out) :: flag
	real(kind=8), intent(out) :: x

	!     If quadratic(ind) is set equal to true in function inip, then
	!     YOU MUST to provide the minimizer x of the associated function.

	!     Assume that quadratic(j) .eq. .true.. Uncomment the following code
	!     and set the minimizer x.

	flag = -1
	
	if ( problem == 2 ) then
			if ( ind == 2 ) then
					x = 1.0d0 / 2.0d-1
					flag = 0
					return
			end if
	end if

! ----------------------------------------------------------------------
	
	if ( problem == 4 ) then
			if ( ind == 3 ) then
					x = 1.0d0 / 2.0d-1
					flag = 0
					return
			end if
	end if

	end subroutine quadstep  

end module myproblem
